﻿using Paliz.Sdk.Tiam;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace TiamSdkSample {
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window {
        ActionWindow prvViewModel;
        public MainWindow() {
            InitializeComponent();
            this.prvViewModel = new ActionWindow();
            this.DataContext = prvViewModel;
        }
    }


    public class ViewModel : INotifyPropertyChanged, ICommand {
        #region Interface Implement
        EventHandler prvCanExecuteChanged;
        PropertyChangedEventHandler prvPropertyChanged;

        event PropertyChangedEventHandler INotifyPropertyChanged.PropertyChanged { add { this.prvPropertyChanged += value; } remove { this.prvPropertyChanged -= value; } }
        event EventHandler ICommand.CanExecuteChanged { add { this.prvCanExecuteChanged += value; } remove { this.prvCanExecuteChanged -= value; } }

        bool ICommand.CanExecute(object parameter) {
            return true;
        }

        void ICommand.Execute(object parameter) {
            var methodInfo = this.GetType().GetMethod(parameter as string);
            if (methodInfo != null) {
                methodInfo.Invoke(this, new object[0]);
            }
        }
        #endregion Interface Implement
        public ViewModel() {
            this.prvDeviceLists = new ObservableCollection<DeviceInfoVM>();
            this.prvUsers = new ObservableCollection<UserVM>();
        }

        private ushort? prvPort;
        public ushort? Port {
            get { return prvPort; }
            set { prvPort = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(Port))); }
        }


        private ObservableCollection<DeviceInfoVM> prvDeviceLists;
        public ObservableCollection<DeviceInfoVM> DeviceLists {
            get { return this.prvDeviceLists; }
            set { this.prvDeviceLists = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(DeviceLists))); }
        }


        private ObservableCollection<UserVM> prvUsers;
        public ObservableCollection<UserVM> Users {
            get { return this.prvUsers; }
            set { this.prvUsers = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(Users))); }
        }
    }

    public class ActionWindow : ViewModel {
        Paliz.Sdk.Tiam.SDKTiam prvSDK;
        public ActionWindow() : base() {
            this.prvSDK = new Paliz.Sdk.Tiam.SDKTiam();
        }
        public void StartServer() {
            if (this.Port.HasValue && this.Port > 0) {
                this.prvSDK.OnConnected += OnConnected;
                this.prvSDK.OnDisconnected += OnDisconnected;
                this.prvSDK.OnNewUserDevice += NewUserInDevice;
                this.prvSDK.OnRealTimeTraffic += RealTimeTraffic;
                this.prvSDK.Initializer(this.Port.Value);
            }
        }

        private bool RealTimeTraffic(Paliz.Sdk.Tiam.ITraffic traffic) {
            return true;
        }

        private void NewUserInDevice(Paliz.Sdk.Tiam.IDevice device, Paliz.Sdk.Tiam.User user) {

        }

        private void OnDisconnected(Paliz.Sdk.Tiam.IDevice device) {

        }

        private bool OnConnected(Paliz.Sdk.Tiam.IDevice device) {
            this.GetDeviceInfo(device);
            return true;
        }

        public async void GetDeviceInfo(Paliz.Sdk.Tiam.IDevice device) {
            var Info = await device.GetDeviceInfoAsync();
            if (Info != null) {
                DeviceInfoVM deviceInfo = this.DeviceLists.FirstOrDefault(n => n.DeviceSerial == device.Serial);
                bool find = deviceInfo == null;
                if (find) {
                    DeviceAction deviceAction = new DeviceAction();
                    deviceAction.OnReceiveTraffic += ReceiveTraffic;
                    deviceAction.OnCleanAllUser += OnCleanAllUser;
                    deviceAction.OnReceiveUser += OnReceiveUser;
                    deviceAction.OnSendUser += OnSendUser;
                    deviceInfo = deviceAction;
                }
                deviceInfo.TrafficCount = Info.TrafficCount;
                deviceInfo.MaxTrafficCount = Info.MaxTrafficStorage;
                deviceInfo.NewTrafficCount = Info.NewTrafficCount;
                deviceInfo.UserCount = Info.UserCount;
                deviceInfo.MaxUserCount = Info.MaxUserStorage;
                deviceInfo.DeviceModel = Info.ModelName;
                deviceInfo.DeviceSerial = device.Serial;
                if (find) {
                    this.DeviceLists.Add(deviceInfo);
                }
            }
        }

        private async void OnSendUser(DeviceInfoVM device) {
            Paliz.Sdk.Tiam.IDevice deviceSDK = this.prvSDK[device.DeviceSerial];
            if (deviceSDK != null) {
                var userSelected = this.Users.Where(n => n.IsSelected).ToList();
                foreach (var user in userSelected) {
                    Paliz.Sdk.Tiam.User tUser = null;
                    if (user.IsPassword) {
                        tUser = new User(null, null, user.Password, null, user.Code, user.Name, null, user.IsAdmin, IdentificationType.Password);
                        await deviceSDK.SetUserInfoAsync(tUser);
                    }
                    if (user.IsCard) {
                        tUser = new User(null, null, null, user.Card, user.Code, user.Name, null, user.IsAdmin, IdentificationType.Card);
                        await deviceSDK.SetUserInfoAsync(tUser);
                    }
                    if (user.IsFingerprint) {
                        int index = 0;
                        foreach (var finger in user.Finger) {
                            tUser = new User(null, finger, null, null, user.Code, user.Name, null, user.IsAdmin, (IdentificationType.RightThumb + index));
                            if (await deviceSDK.SetUserInfoAsync(tUser))
                                index++;
                        }
                    }
                    if (user.IsFace) {
                        tUser = new User(user.Face, null, null,  null, user.Code, user.Name, null, user.IsAdmin, IdentificationType.Walkthrough);
                        await deviceSDK.SetUserInfoAsync(tUser);
                    }
                }
            }
        }

        private async void OnReceiveUser(DeviceInfoVM device) {
            Paliz.Sdk.Tiam.IDevice deviceSDK = this.prvSDK[device.DeviceSerial];
            if (deviceSDK != null) {
                this.Users.Clear();
                var users = await deviceSDK.GetUserListsAsync();
                if (users != null) {
                    while (users.IsAwaitable ? await users : users.MoveNext()) {
                        var user = this.Users.FirstOrDefault(n => n.Code == users.Current.UserID);
                        if (user != null) {
                            user.IdentificationTypes.Add(users.Current.IdentificationType);
                            user.Update();
                        }
                        else {
                            user = new UserAction() {
                                Code = users.Current.UserID,
                                Name = users.Current.Name,
                                IsAdmin = users.Current.Admin,
                                DeviceInfo = device
                            };
                            (user as UserAction).OnGetPassword += OnGetPassword;
                            (user as UserAction).OnGetCard += OnGetCard;
                            (user as UserAction).OnGetFingerprint += OnGetFingerprint;
                            (user as UserAction).OnGetFace += OnGetFace;
                            user.IdentificationTypes.Add(users.Current.IdentificationType);
                            this.Users.Add(user);
                        }
                    }
                }
            }
        }

        private async void OnGetFace(UserVM user) {
            Paliz.Sdk.Tiam.IDevice deviceSDK = this.prvSDK[user.DeviceInfo.DeviceSerial];
            if (deviceSDK != null) {
                var info = await deviceSDK.GetUserInfoAsync(user.Code, IdentificationType.Walkthrough);
                if (info != null) {
                    user.Face = info.Face;
                    user.Update();
                }
            }
        }

        private async void OnGetFingerprint(UserVM user) {
            Paliz.Sdk.Tiam.IDevice deviceSDK = this.prvSDK[user.DeviceInfo.DeviceSerial];
            if (deviceSDK != null) {
                int count = user.IdentificationTypes.Count(n => n >= IdentificationType.RightThumb && n <= IdentificationType.LeftLittle);
                user.Finger = new byte[count][];
                count = 0;
                foreach (var Identification in user.IdentificationTypes) {
                    if (Identification >= IdentificationType.RightThumb && Identification <= IdentificationType.LeftLittle) {
                        var info = await deviceSDK.GetUserInfoAsync(user.Code, Identification);
                        if (info != null) {
                            user.Finger[count] = info.Finger;
                            count++;
                        }
                    }
                }
                user.Update();
            }
        }

        private async void OnGetCard(UserVM user) {
            Paliz.Sdk.Tiam.IDevice deviceSDK = this.prvSDK[user.DeviceInfo.DeviceSerial];
            if (deviceSDK != null) {
                var info = await deviceSDK.GetUserInfoAsync(user.Code, IdentificationType.Card);
                if (info != null) {
                    user.Card = info.Card;
                    user.Update();
                }
            }
        }

        private async void OnGetPassword(UserVM user) {
            Paliz.Sdk.Tiam.IDevice deviceSDK = this.prvSDK[user.DeviceInfo.DeviceSerial];
            if (deviceSDK != null) {
                var info = await deviceSDK.GetUserInfoAsync(user.Code, IdentificationType.Password);
                if (info != null) {
                    user.Password = info.Password;
                    user.Update();
                }
            }
        }

        private async void OnCleanAllUser(DeviceInfoVM device) {
            Paliz.Sdk.Tiam.IDevice deviceSDK = this.prvSDK[device.DeviceSerial];
            if (deviceSDK != null) {
                var userSelected = this.Users.Where(n => n.IsSelected).ToList();
                if (userSelected.Count > 0) {
                    string userDelete = string.Empty;
                    foreach (var user in userSelected) {
                        if (await deviceSDK.DeleteUserAsync(new User(user.Code, IdentificationType.All))) {
                            userDelete += $"{user.Card} - ";
                        }
                    }
                    userDelete += "Deleted From Device";
                    MessageBox.Show(userDelete);
                }
                else if (await deviceSDK.CleanUserAsync()) {
                    MessageBox.Show("Device Is Clear");
                }
            }
        }

        private async void ReceiveTraffic(DeviceInfoVM device) {
            Paliz.Sdk.Tiam.IDevice deviceSDK = this.prvSDK[device.DeviceSerial];
            if (deviceSDK != null) {
                var traffics = await deviceSDK.GetAllLogsAsync(null, null);
                if (traffics != null) {
                    var trafficsWindow = new Traffics(traffics);
                    trafficsWindow.ShowDialog();
                }
            }
        }
    }


    public class DeviceInfoVM : INotifyPropertyChanged, ICommand {
        #region Interface Implement
        EventHandler prvCanExecuteChanged;
        PropertyChangedEventHandler prvPropertyChanged;

        event PropertyChangedEventHandler INotifyPropertyChanged.PropertyChanged { add { this.prvPropertyChanged += value; } remove { this.prvPropertyChanged -= value; } }
        event EventHandler ICommand.CanExecuteChanged { add { this.prvCanExecuteChanged += value; } remove { this.prvCanExecuteChanged -= value; } }

        bool ICommand.CanExecute(object parameter) {
            return true;
        }

        void ICommand.Execute(object parameter) {
            var methodInfo = this.GetType().GetMethod(parameter as string);
            if (methodInfo != null) {
                methodInfo.Invoke(this, new object[0]);
            }
        }
        #endregion Interface Implement

        private int prvTrafficCount;
        public int TrafficCount {
            get { return prvTrafficCount; }
            set { prvTrafficCount = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(TrafficCount))); }
        }

        private int prvMaxTrafficCount;
        public int MaxTrafficCount {
            get { return this.prvMaxTrafficCount; }
            set { this.prvMaxTrafficCount = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(MaxTrafficCount))); }
        }

        private int prvUserCount;
        public int UserCount {
            get { return this.prvUserCount; }
            set { this.prvUserCount = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(UserCount))); }
        }

        private int prvMaxUserCount;
        public int MaxUserCount {
            get { return this.prvMaxUserCount; }
            set { this.prvMaxUserCount = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(MaxUserCount))); }
        }

        private int prvNewTrafficCount;
        public int NewTrafficCount {
            get { return this.prvNewTrafficCount; }
            set { this.prvNewTrafficCount = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(NewTrafficCount))); }
        }

        private string prvDeviceModel;
        public string DeviceModel {
            get { return this.prvDeviceModel; }
            set { this.prvDeviceModel = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(DeviceModel))); }
        }

        private string prvDeviceSerial;
        public string DeviceSerial {
            get { return this.prvDeviceSerial; }
            set { this.prvDeviceSerial = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(DeviceSerial))); }
        }
    }


    public delegate void DAC(DeviceInfoVM device);
    public delegate void UAC(UserVM user);

    public class DeviceAction : DeviceInfoVM {

        public void ReceiveTraffic() {
            this.OnReceiveTraffic?.Invoke(this);
        }
        public void ReceiveUser() {
            this.OnReceiveUser?.Invoke(this);
        }
        public void SendUser() {
            this.OnSendUser?.Invoke(this);

        }
        public void CleanAllUser() {
            this.OnCleanAllUser?.Invoke(this);
        }

        public event DAC OnReceiveTraffic;
        public event DAC OnReceiveUser;
        public event DAC OnSendUser;
        public event DAC OnCleanAllUser;
    }

    public class UserVM : INotifyPropertyChanged, ICommand {
        #region Interface Implement
        EventHandler prvCanExecuteChanged;
        PropertyChangedEventHandler prvPropertyChanged;

        event PropertyChangedEventHandler INotifyPropertyChanged.PropertyChanged { add { this.prvPropertyChanged += value; } remove { this.prvPropertyChanged -= value; } }
        event EventHandler ICommand.CanExecuteChanged { add { this.prvCanExecuteChanged += value; } remove { this.prvCanExecuteChanged -= value; } }

        bool ICommand.CanExecute(object parameter) {
            return true;
        }

        void ICommand.Execute(object parameter) {
            var methodInfo = this.GetType().GetMethod(parameter as string);
            if (methodInfo != null) {
                methodInfo.Invoke(this, new object[0]);
            }
        }
        #endregion Interface Implement

        public UserVM() {
            this.IdentificationTypes = new List<IdentificationType>();
        }

        public DeviceInfoVM DeviceInfo { get; set; }
        public List<Paliz.Sdk.Tiam.IdentificationType> IdentificationTypes { get; set; }

        private long prvCode;
        public long Code {
            get { return this.prvCode; }
            set { this.prvCode = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(Code))); }
        }


        private string prvName;
        public string Name {
            get { return this.prvName; }
            set { this.prvName = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(Name))); }
        }


        private bool prvIsAdmin;
        public bool IsAdmin {
            get { return this.prvIsAdmin; }
            set { this.prvIsAdmin = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(IsAdmin))); }
        }


        private bool prvIsSelected;
        public bool IsSelected {
            get { return this.prvIsSelected; }
            set { this.prvIsSelected = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(IsSelected))); }
        }

        public bool IsPassword {
            get { return this.IdentificationTypes.Any(n => n == IdentificationType.Password) && !this.Password.HasValue; }
        }

        public bool IsCard {
            get { return this.IdentificationTypes.Any(n => n == IdentificationType.Card) && !this.Card.HasValue; }
        }

        public bool IsFingerprint {
            get { return this.IdentificationTypes.Any(n => n >= IdentificationType.RightThumb && n <= IdentificationType.LeftLittle) && this.Finger == null; }
        }

        public bool IsFace {
            get { return this.IdentificationTypes.Any(n => n == IdentificationType.Walkthrough) && this.Face == null; }
        }

        public int? Password { get; set; }
        public long? Card { get; set; }
        public byte[][] Finger { get; set; }
        public byte[] Face { get; set; }

        public void Update() {
            if (this.IdentificationTypes.Any(n => n == IdentificationType.Password)) {
                this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(IsPassword)));
            }

            if (this.IdentificationTypes.Any(n => n == IdentificationType.Card)) {
                this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(IsCard)));
            }

            if (this.IdentificationTypes.Any(n => n >= IdentificationType.RightThumb && n <= IdentificationType.LeftLittle)) {
                this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(IsFingerprint)));
            }

            if (this.IdentificationTypes.Any(n => n == IdentificationType.Password)) {
                this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(IsFace)));
            }
        }
    }

    public class UserAction : UserVM {
        public void GetPassword() {
            this.OnGetPassword?.Invoke(this);
        }
        public void GetCard() {
            this.OnGetCard?.Invoke(this);
        }
        public void GetFingerprint() {
            this.OnGetFingerprint?.Invoke(this);
        }
        public void GetFace() {
            this.OnGetFace?.Invoke(this);
        }
        public event UAC OnGetPassword;
        public event UAC OnGetCard;
        public event UAC OnGetFingerprint;
        public event UAC OnGetFace;
    }
    public class AdminConverter : IValueConverter {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture) {
            if (value == null) return Properties.Resources.False;
            if ((value as bool?) == true) return Properties.Resources.True;
            return Properties.Resources.False;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture) {
            throw new NotImplementedException();
        }
    }
}
