﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace TiamSdkSample {
    /// <summary>
    /// Interaction logic for Traffics.xaml
    /// </summary>
    public partial class Traffics : Window {
        private TrafficsViewModel prvViewModel;
        public Traffics() {
            InitializeComponent();
            this.prvViewModel = new TrafficsViewModel();
            this.DataContext = this.prvViewModel;
        }
        public Traffics(Paliz.Sdk.Tiam.IAwaiterCollection<Paliz.Sdk.Tiam.ITraffic> traffics) : this() {
            this.prvViewModel.Load(traffics);
        }
    }

    public class TrafficsViewModel : INotifyPropertyChanged, ICommand {
        #region Interface Implement
        EventHandler prvCanExecuteChanged;
        PropertyChangedEventHandler prvPropertyChanged;

        event PropertyChangedEventHandler INotifyPropertyChanged.PropertyChanged { add { this.prvPropertyChanged += value; } remove { this.prvPropertyChanged -= value; } }
        event EventHandler ICommand.CanExecuteChanged { add { this.prvCanExecuteChanged += value; } remove { this.prvCanExecuteChanged -= value; } }

        bool ICommand.CanExecute(object parameter) {
            return true;
        }

        void ICommand.Execute(object parameter) {
            var methodInfo = this.GetType().GetMethod(parameter as string);
            if (methodInfo != null) {
                methodInfo.Invoke(this, new object[0]);
            }
        }
        #endregion Interface Implement

        public TrafficsViewModel() {
            this.prvTraffics = new ObservableCollection<TrafficItemVM>();
        }

        private ObservableCollection<TrafficItemVM> prvTraffics;
        public ObservableCollection<TrafficItemVM> Traffics {
            get { return this.prvTraffics; }
            set { this.prvTraffics = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(Traffics))); }
        }

        public async void Load(Paliz.Sdk.Tiam.IAwaiterCollection<Paliz.Sdk.Tiam.ITraffic> traffics) {
            while (traffics.IsAwaitable?await traffics:traffics.MoveNext()) {
                TrafficItemVM trafficItem = new TrafficItemVM() {
                    Code=traffics.Current.UserID,
                    Event=traffics.Current.Event,
                    Date=traffics.Current.Date,
                    InOut=traffics.Current.InOut,
                    Mode=traffics.Current.Mode,
                    Image=traffics.Current.Image
                };
                this.Traffics.Add(trafficItem);
            }
        }
    }

    public class TrafficItemVM : INotifyPropertyChanged, ICommand {
        #region Interface Implement
        EventHandler prvCanExecuteChanged;
        PropertyChangedEventHandler prvPropertyChanged;

        event PropertyChangedEventHandler INotifyPropertyChanged.PropertyChanged { add { this.prvPropertyChanged += value; } remove { this.prvPropertyChanged -= value; } }
        event EventHandler ICommand.CanExecuteChanged { add { this.prvCanExecuteChanged += value; } remove { this.prvCanExecuteChanged -= value; } }

        bool ICommand.CanExecute(object parameter) {
            return true;
        }

        void ICommand.Execute(object parameter) {
            var methodInfo = this.GetType().GetMethod(parameter as string);
            if (methodInfo != null) {
                methodInfo.Invoke(this, new object[0]);
            }
        }
        #endregion Interface Implement


        private long prvCode;
        public long Code {
            get { return this.prvCode; }
            set { this.prvCode = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(Code))); }
        }

        private Paliz.Sdk.Tiam.IdentificationMode prvMode;
        public Paliz.Sdk.Tiam.IdentificationMode Mode {
            get { return this.prvMode; }
            set { this.prvMode = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(Mode))); }
        }


        private DateTime prvDate;
        public DateTime Date {
            get { return this.prvDate; }
            set { this.prvDate = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(Date))); }
        }

        private int prvInOut;
        public int InOut {
            get { return this.prvInOut; }
            set { this.prvInOut = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(InOut))); }
        }


        private int prvEvent;
        public int Event {
            get { return this.prvEvent; }
            set { this.prvEvent = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(Event))); }
        }

        private byte[] prvImage;
        public byte[] Image {
            get { return this.prvImage; }
            set { this.prvImage = value; this.prvPropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(Image))); }
        }
    }
}
